#!/usr/bin/env python
"""This file is automatically generated by generate_version_info
It uses the current working tree to determine the revision.
So don't edit it. :)
"""

version_info = {'branch_nick': u'wicd',
 'build_date': '2012-04-30 21:26:29 +0200',
 'clean': None,
 'date': '2012-04-30 21:26:10 +0200',
 'revision_id': 'd.paleino@gmail.com-20120430192610-0eblawm4gmxvwn44',
 'revno': '768'}

revisions = {}

file_revisions = {}


if __name__ == '__main__':
    print 'revision: %(revno)s' % version_info
    print 'nick: %(branch_nick)s' % version_info
    print 'revision id: %(revision_id)s' % version_info
